import std/lists
import std/strutils
import std/strformat

import types
import convenience

proc openAndWrite*(fileName: string, gcodeStates: DoublyLinkedList[
    GcodeState]): bool =
  ## Writes a file from the state stored in gcode
  let f = open(fileName, fmWrite)
  defer: f.close()

  # Initialize something to check the state against between iterations
  var old: GcodeState
  for line in gcodeStates:
    if not line.comment.isEmptyOrWhitespace:
        # Comments have the highest priority going out as they have the lowest priority
        # going in. This means it must only contain information that was not processable
        # and therefor should be written back raw.
        f.writeLine(line.comment)
    if not line.parsedComment.isEmptyOrWhitespace:
        # Even parsed comments should be added back for other tools to read
        f.writeLine(line.parsedComment)
    elif line.posX != old.posX or line.posY != old.posY or line.posZ != old.posZ or line.speed != old.speed:
      # Motion is required
      var moveX: string
      var moveY: string
      if line.posX != old.posX or line.posY != old.posY:
        # Motion on X/Y has happened. Set both
        moveX = if line.posX != old.posX: fmt" X{line.posX.formatMove}" else: fmt" X{old.posX.formatMove}"
        moveY = if line.posY != old.posY: fmt" Y{line.posY.formatMove}" else: fmt" Y{old.posY.formatMove}"
      let moveZ = if line.posZ != old.posZ: fmt" Z{line.posZ.formatMove}" else: ""
      # Set move speed if it's not 0
      var speedStr: string
      if line.speed != 0.0:
        if line.moveType == MoveType.travel or line.moveType == MoveType.layerChange:
            speedStr = fmt" F{line.speed.formatMove}"
        else: speedStr = fmt" E{line.speed.formatMove}"

      f.writeLine(fmt"G1{moveX}{moveY}{moveZ}{speedStr}")

    elif line.comment.isEmptyOrWhitespace: f.writeLine(line.comment)

    else:
      discard
    old = line
  return true
