type
  Feature* = enum
    ## Contains of all features avaliable
    externalPerimeter = "External perimeter",
    internalPerimeter = "Internal perimeter",
    overhangPerimeter = "Overhang perimeter",
    topSolidInfill = "Top solid infill"
    solidInfill = "Solid infill",
    internalInfill = "Internal infill"
    bridgeInfill = "Bridge infill",
    internalBridgeInfill = "Internal bridge infill"
    thinWall = "Thin wall",
    gapFill = "Gap fill"
    skirt = "Skirt",
    supportMaterial = "Support material",
    supportMaterialInterface = "Support material interface",
    custom = "Custom"
    other = "Other"

type
  MoveType* = enum
    ## All move types avaliable
    travel, extrude, layerChange

type
  BaseGcode* = enum
    ## All base gcodes avaliable in most firmwares
    move = "G0"
    moveAlt = "G1"
    absolute = "G90"
    relative = "G91"
    extruderTemp = "M104"
    bedTemp = "M140"

type
  KlipperAccels* = tuple
    ## List of all accels for features. This is meant to be used for
    ## Klipper firmware. Formatted like
    ## `SET_VELOCITY_LIMIT ACCEL=8000 ACCEL_TO_DECEL=8000 SQUARE_CORNER_VELOCITY=8`
    externalPerimeter: string
    internalPerimeter: string
    overhangPerimeter: string
    topSolidInfill: string
    solidInfill: string
    internalInfill: string
    bridgeInfill: string
    internalBridgeInfill: string
    thinWall: string
    gapFill: string
    skirt: string
    supportMaterial: string
    supportMaterialInterface: string
    custom: string
    other: string
    firstLayer: string
    travel: string

type
  GcodeState* = tuple
    ## The entire gcode state is stored here. This should be enough data
    ## to reconstruct a usable generic gcode file for most firmwares
    preInject: string
    moveMode: BaseGcode
    moveType: MoveType
    feature: Feature
    layer: int
    posX: float64
    posY: float64
    posZ: float64
    speed: float64
    comment: string
    parsedComment: string
    postInject: string
    klipperAccels: KlipperAccels
    extruderTemp: float
    bedTemp: float

type
  BlindGcode* = enum
    ## Known, but not manipulated gcodes. These are blindly copied
    ## and treated like comments
    retract = "G10"
    unretract = "G11"
    setUnitMM = "G21"
    home = "G28"
    setPosition = "G92"
    buildPercentage = "M73"
    setExtruderAbsolute = "M82"
    setExtruderRelative = "M83"
    fanOff = "M107"
    fanOn = "M106"
    displayMessage = "M117"

type
  KlipperBlindGcode* = enum
    ## Known, and "unsupported" klipper gcodes. These are either user
    ## defined, or just not supported yet.
    printStart = "PRINT_START"
    printEnd = "PRINT_END"
    setPressureAdvance = "SET_PRESSURE_ADVANCE"
    timelapseTakeFrame = "TIMELAPSE_TAKE_FRAME"
